@@ -15,6 +15,11 @@ from blueman.Functions import have, mask_ip4_address, dprint, is_running
 from _blueman import create_bridge, destroy_bridge, BridgeException
 from subprocess import call, Popen
 
+
+class NetworkSetupError(Exception):
+    pass
+
+
 def calc_ip_range(ip):
     '''Calculate the ip range for dhcp config'''
     start_range = bytearray(ip)
@@ -66,7 +71,7 @@ class DnsMasqHandler(object):
                 dprint("pid", self.pid)
                 self.netconf.lock("dhcp")
             else:
-                raise Exception("dnsmasq failed to start. Check the system log for errors")
+                raise NetworkSetupError("dnsmasq failed to start. Check the system log for errors")
 
 
     def do_remove(self):
@@ -174,7 +179,7 @@ class DhcpdHandler(object):
                 dprint("pid", self.pid)
                 self.netconf.lock("dhcp")
             else:
-                raise Exception("dhcpd failed to start. Check the system log for errors")
+                raise NetworkSetupError("dhcpd failed to start. Check the system log for errors")
 
 
     def do_remove(self):
@@ -305,13 +310,25 @@ class NetConf(object):
         ip_str = inet_ntoa(self.ip4_address)
         mask_str = inet_ntoa(self.ip4_mask)
 
-        if self.ip4_changed or not self.locked("ifconfig"):
+        if self.ip4_changed or not self.locked("netconfig"):
             self.enable_ip4_forwarding()
 
-            ret = call(["ifconfig", "pan1", ip_str, "netmask", mask_str, "up"])
-            if ret != 0:
-                raise Exception("Failed to setup interface pan1")
-            self.lock("ifconfig")
+            if have("ip"):
+                ret = call(["ip", "link", "set", "dev", "pan1", "up"])
+                if ret != 0:
+                    raise NetworkSetupError("Failed to bring up interface pan1")
+                ret = call(["ip", "address", "add", "/".join((ip_str, mask_str)), "dev", "pan1"])
+                if ret != 0:
+                    raise NetworkSetupError("Failed to add ip address %s with netmask %s" % (ip_str, mask_str))
+            elif have('ifconfig'):
+                ret = call(["ifconfig", "pan1", ip_str, "netmask", mask_str, "up"])
+                if ret != 0:
+                    raise NetworkSetupError("Failed to add ip address %s with netmask %s" % (ip_str, mask_str))
+            else:
+                raise NetworkSetupError(
+                    "Neither ifconfig or ip commands are found. Please install net-tools or iproute2")
+
+            self.lock("netconfig")
 
         if self.ip4_changed or not self.locked("iptables"):
             self.del_ipt_rules()
@@ -338,7 +355,7 @@ class NetConf(object):
             destroy_bridge("pan1")
         except:
             pass
-        self.unlock("ifconfig")
+        self.unlock("netconfig")
 
         self.del_ipt_rules()
 
